@@ -47,8 +47,8 @@ namespace jellyfish {
       header() { }
       explicit header(char *ptr) {
         if(memcmp(ptr, file_type, sizeof(type)))
-          eraise(ErrorReading) << "Bad file type '" << err::substr(ptr, sizeof(type))
-                              << "', expected '" << err::substr(file_type, sizeof(type)) << "'";
+          throw ErrorReading(err::msg()  << "Bad file type '" << err::substr(ptr, sizeof(type))
+                              << "', expected '" << err::substr(file_type, sizeof(type)) << "'");
         memcpy((void *)this, ptr, sizeof(struct header));
       }
     };
@@ -185,22 +185,22 @@ namespace jellyfish {
         io = new std::ifstream(filename.c_str());
         io->read((char *)&header, sizeof(header));
         if(!io->good())
-          eraise(ErrorReading) << "'" << filename << "': " 
-                               << "File truncated";
+          throw ErrorReading(err::msg()  << "'" << filename << "': " 
+                               << "File truncated");
         if(memcmp(header.type, file_type, sizeof(header.type)))
-          eraise(ErrorReading) << "'" << filename << "': " 
+          throw ErrorReading(err::msg()  << "'" << filename << "': " 
                                << "Bad file type '" 
                               << err::substr(header.type, sizeof(header.type)) << "', expected '"
-                              << err::substr(file_type, sizeof(header.type)) << "'";
+                              << err::substr(file_type, sizeof(header.type)) << "'");
 
         if(header.key_len > 64 || header.key_len == 0)
-          eraise(ErrorReading) << "'" << filename << "': " 
+          throw ErrorReading(err::msg()  << "'" << filename << "': " 
                                << "Invalid key length '"
-                               << header.key_len << "'";
+                               << header.key_len << "'");
         if(header.size != (1UL << floorLog2(header.size)))
-          eraise(ErrorReading) << "'" << filename << "': " 
+          throw ErrorReading(err::msg()  << "'" << filename << "': " 
                                << "Size '" << header.size 
-                               << "' is not a power of 2";
+                               << "' is not a power of 2");
         key_len  = (header.key_len / 8) + (header.key_len % 8 != 0);
         record_len = key_len + header.val_len;
         buffer_len = record_len * (_buff_len / record_len);
@@ -215,10 +215,10 @@ namespace jellyfish {
           std::streamoff list_size = get_file_size(*io);
           if(list_size != (std::streamoff)-1 &&
              list_size - (header.distinct * record_len) != 0) {
-            eraise(ErrorReading) << "'" << filename << "': " 
+            throw ErrorReading(err::msg()  << "'" << filename << "': " 
                                  << "Bad hash size '" << list_size 
                                  << "', expected '"
-                                 << (header.distinct * record_len) << "' bytes";
+                                 << (header.distinct * record_len) << "' bytes");
           }
         }
         key = val = 0;
@@ -322,9 +322,9 @@ namespace jellyfish {
         canonical(false)
       {
         if(header.distinct != 0 && file.end() - base - header.distinct * record_len != 0)
-          eraise(ErrorReading) << "'" << file.path() << "': " 
+          throw ErrorReading(err::msg()  << "'" << file.path() << "': " 
                                << "Bad hash size '" << (file.end() - base)
-                               << "', expected '" << header.distinct * record_len << "' bytes";
+                               << "', expected '" << header.distinct * record_len << "' bytes");
           
         get_key(0, &first_key);
         first_pos = get_pos(first_key);
@@ -346,9 +346,9 @@ namespace jellyfish {
         canonical(false)
       { 
         if(header.distinct != 0 && file.end() - base - header.distinct * record_len != 0)
-          eraise(ErrorReading) << "'" << file.path() << "': " 
+          throw ErrorReading(err::msg()  << "'" << file.path() << "': " 
                                << "Bad hash size '" << (file.end() - base)
-                               << "', expected '" << header.distinct * record_len << "' bytes";
+                               << "', expected '" << header.distinct * record_len << "' bytes");
           
         get_key(0, &first_key);
         first_pos = get_pos(first_key);
