@@ -23,7 +23,7 @@ namespace jellyfish {
     size_t  _size;
     T      *_data;
   public:
-    simple_growing_array(size_t capacity = 100) : 
+    explicit simple_growing_array(size_t capacity = 100) : 
       _capacity(capacity / 2), _size(0), _data(0) { resize(); }
 
     virtual ~simple_growing_array() {
@@ -52,7 +52,7 @@ namespace jellyfish {
         free(ndata);
         _data = 0;
         _capacity = _capacity / 2;
-        eraise(SimpleGrowingArrayError) << "Out of memory" << err::no;
+        throw SimpleGrowingArrayError(err::msg()  << "Out of memory" << err::no);
       }
       _data = (T*)ndata;
     }
